
# q-expansion of newform 450.8.a.g, downloaded from the LMFDB on 22 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 450
weight = 8
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-8], [0], [0], [232], [60], [-14054], [13938], [53564], [-27000], [88554], [-120352], [-469646], [510246], [-145412], [304560], [94398], [1885740], [-271690], [-1074668], [2505480], [2954566], [-7354768], [-3846756], [-5685162], [-7981346], [-15462798], [20902408], [20564484], [13985030], [26443794], [9340000], [44679540], [-56992902], [30682244], [-28362846], [-34565272], [-50066006], [1633588], [-99481800], [-100775034], [-85916316], [-7349506], [-25552896], [-136814786], [-138052818], [6204152], [-165041860], [54311680], [133927884], [20984270], [12168474], [46923504], [-229326958], [-450379476], [66525186], [-110722920], [131053914], [-566889616], [-130574366], [270368022], [91107436], [-622671282], [113281540], [-378343560], [111400246], [-670098186], [-544095868], [-962373938], [-592828524], [-973267690], [1591265442], [-837928344], [1144475344], [-1366732862], [-491619436], [532012320], [-683512686], [1468424986], [-708291954], [2136030314], [-1221300780], [-117753970], [2927493360], [922065838], [-2694312568], [1509278964], [-794655426], [-1822702778], [179582490], [-395890832], [570281052], [-4368232224], [-329328056], [2546657436], [1424952284], [-2183971032], [5256234954], [-3194805498], [1922192860], [1623521750], [876823924], [-229688826], [786151356], [-3300875082], [-1540329580], [3797687614], [8296760484], [4931956530], [-6511516584], [-4174601302], [-5469791168], [7927644466], [-9702766950], [534840356], [-8981448568], [-681488130], [3680069908], [9032687832], [2904771366], [288225348], [4043262494], [-6389004002], [-10825631538], [881962116], [5368826204], [-4373411574], [4603831022], [-5783244720], [-14287082696], [11889570154], [14199256172], [-8295394248], [-10733403760], [-1283887166], [-11399124618], [-12638828542], [17893523694], [-16508481308], [-1820520810], [21744506406], [-19386943324], [11656477122], [4640543320], [10060358196], [15625564982], [28355659464], [-15207182558], [26977599786], [-15996420556], [22264678848], [-20835862406], [-24136328466], [-8171266172], [26472630120], [-38488001252], [8734296528], [19169501096], [-9513520866], [20987145382], [-1923017478], [15248286588], [-16005149238], [30525365032], [-38573240964], [-19080515406], [-48833018808], [38373184544], [22252065202], [19056986258], [6117651582], [-54710653332], [6570144374], [28081002696], [31389013510], [55635493424], [55125337878], [22617836468], [16993625778], [-47146082744], [66534910406], [13110393376], [-68116370508], [-5813598830], [-29084567622], [37309411920], [-60237977886], [8001710314], [-1251423116], [-53124682726], [1283767104], [18529408894], [-36226303836], [54393520892], [47714826090], [85730522892], [65576140122], [8099613650], [26244679882], [24644411970], [5365873752], [-25076492262], [13549128368], [-57349698014], [-33900036574], [-16715086500], [85218423990], [-105867302080], [-43518146964], [97484813190], [23798548292], [5963117326], [124510594338], [-19681704008], [-125629613100], [105205995432], [72155088986], [-130496791280], [181231273806], [-111681575928], [-47760717738], [63431691470], [105253945160], [14487556350], [-22218710096], [73678840476], [155591506718], [127061979882], [113440488480], [132462117640], [107582035932], [-137686760774], [40132139420], [37578473024], [-33908850618], [60265947676], [-60930638832], [-169618684750], [37766408862], [-204111127092], [-108139720152], [-42053671428], [-260923377580], [-39904075736], [-19162904218], [140910567282], [-135034603368], [-206365080704], [-234072358764], [-71621612572], [17235751152], [43110705226], [-153776215746], [85904362776], [136501252154], [-91786943898], [188295003012], [-264661618594], [210049135084], [-215035332594], [-179674823306], [-257687210720], [275859422124], [66734325038], [219749151658], [76380629346], [-112659049684], [203622813690], [-147135901002], [-92082210548], [167436480558], [31052254886], [-332638493660], [399391990102], [356576776160], [-85217498258], [11839448344], [-372398714508], [-208272981898], [-142599857542], [-9783576060], [50493742464], [-140870263720], [-164093710296], [475324494830], [132806656732], [77542832016], [-465297272498], [57845319966], [64864994792], [556503909726], [145007605050], [30384222780], [126193192458], [-206612743620], [296374835098], [-63930408342], [-46514312224], [10997863614], [624639807660], [191677695124], [-235502449850], [-421878971034], [-606004210960], [525968669724], [-547110618892], [581214100318], [-26509889916], [392723125766], [510763907448], [542404684882], [-535977720816], [-461650655454], [478303023774], [-255741444044], [-162749025096], [-629993971366], [-189725515932], [-426789346944], [-236536548290], [276503974350], [-277473557956], [284126997334], [-636268491606], [-642783994112], [217833580698], [600191466002], [221381114060], [-116334832148], [383582121024], [-60228712242], [821064241862], [-423609517962], [334184576768], [-919960406100], [-361646086012], [729887772948], [-264880982506], [-345881698014], [-1034827876966], [-244155848240], [-256783069118], [-531974920566], [289354076562], [-895833101320], [-939373445994], [684963043284], [258653658638], [-590128377668], [43665346032], [1015153669566], [190054971980], [374930682310], [607484286618], [-61569705104], [1327602696926], [649394793258], [-704330822688], [-724742335332], [-583625101038], [362433849384], [864514595410], [-1279413681402], [801881194320], [70347387660], [-873550252556], [292774315174], [-1345674807930], [445457212264], [-305444920534], [-241360884396], [-203911144924], [636090406704], [-1124540654718], [-1410343219960], [-921479541494], [-1601887543356], [-603834259488], [-392027042018], [624590329902], [-1202094059978], [-263756265078], [460593402042], [769008699064], [735661731234], [925363904492], [1352267031864], [-922062140272], [-380783657918], [-548620907444], [-1371979793952], [-780086204542], [-434543332242], [50906171772], [560186401636], [142395342936], [-224251206506], [440837869088], [-446132429658], [-1661621980], [979213281474], [1573741674806], [-22870165950], [1155809155504], [463430759082], [-336033314190], [-1202521740136], [198512990458], [1011226953966], [-2241849505532], [537877510452], [224487542926], [1689183672798], [325684268820], [2331752005484], [-916907256026], [-993556451334], [1728032334144], [-836074398296], [169750495794], [110032329672], [-1841812849878], [-1947037274318], [-254762236944], [1202554823148], [-940976829434], [431744754726], [442310608092], [332863421334], [-800452510540], [-50754102984]]
